//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Collections.Specialized;
using System.Xml;
using System.Windows.Forms;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.ObjectMaintenance.Forms;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.CustomControls;

namespace Digitaldeposit.Ssbm.ObjectMaintenance.UserControls
{
	/// <summary>
	/// Takes care of all the UI related operation to create a Database Certificate object.
	/// </summary>
	public partial class CertificateMaintenance : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		public CertificateMaintenance(CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			InitializeComponent();
			
			//Prepare the form.
			Setup(commonForm);
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from form to populate Certificate object with corresponding UI data.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <param name="additionalReturnValues">Some of the values cannot be set to the return Certificate object, so add them to this hashtable.</param>
		/// <returns>A populated Certificate object which can then undergo any SQL Operation like Create() or Alter()</returns>
		public Certificate Populate(CommonForm commonForm, ref Hashtable additionalReturnValues)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "commonForm", "Certificate Maintenance"));

			if (additionalReturnValues == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "additionalReturnValues", "Certificate Maintenance"));

			Certificate cert;

			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
			{
				lock (commonForm.Database.Certificates.SyncRoot)
				{
					cert = commonForm.Database.Certificates[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
				}
			}
			else
			{
				cert = new Certificate();
				cert.Parent = commonForm.Database;
				cert.Name = txtName.Text;
			}

			cert.ActiveForServiceBrokerDialog = chkActiveForSbDialog.Checked;

			//Add the following values to additionalReturnValues, since it cannot be set on "Certificate" object.
			//Encryption Text, Decryption Text, Certificate Path, Certificate PrivatekeyPath
			additionalReturnValues.Clear();
			additionalReturnValues.Add(ServiceBrokerConstants.CertEncryptionText, txtEncryptPassword.Text);
			additionalReturnValues.Add(ServiceBrokerConstants.CertDecryptionText, txtDecryptPassword.Text);
			additionalReturnValues.Add(ServiceBrokerConstants.CertPath, txtCertFilePath.Text);
			additionalReturnValues.Add(ServiceBrokerConstants.CertPrivateKeyPath, txtCertPrivateKeyFilePath.Text);
			additionalReturnValues.Add(ServiceBrokerConstants.CertUseMasterKey, chkMasterKey.Checked);

			switch ((CertificateSource)cmbCertSource.SelectedItem)
			{
				case CertificateSource.SelfSigned:
					cert.Subject = txtCertSubject.Text;
					cert.StartDate = txtStartDate.Value;
					cert.ExpirationDate = txtEndDate.Value;
					additionalReturnValues.Add(ServiceBrokerConstants.CertSource, CertificateSource.SelfSigned);
					break;
				case CertificateSource.FromFileWithPrivateKey:
					additionalReturnValues.Add(ServiceBrokerConstants.CertSource, CertificateSource.FromFileWithPrivateKey);
					break;
				case CertificateSource.FromFile:
					additionalReturnValues.Add(ServiceBrokerConstants.CertSource, CertificateSource.FromFile);
					break;
			}

			if (cmbAuthorization.Text.Length != 0)
				cert.Owner = cmbAuthorization.Text;

			if (ValidateControls())
			{
				return cert;
			}
			return null;
		}
		#endregion

		#region Event Handlers
		private void cmbCertSource_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch ((CertificateSource)cmbCertSource.SelectedItem)
			{
				case CertificateSource.SelfSigned:
					panelSelfSigned.Enabled = true;
					panelFile.Enabled = false;
					break;
				case CertificateSource.FromFileWithPrivateKey:
					panelSelfSigned.Enabled = false;
					panelFile.Enabled = true;
					break;
				case CertificateSource.FromFile:
					panelSelfSigned.Enabled = false;
					panelFile.Enabled = true;
					txtCertPrivateKeyFilePath.Enabled = false;
					txtDecryptPassword.Enabled = false;
					break;
			}

		}
		private void btnBrowseCertFile_Click(object sender, EventArgs e)
		{
			this.openFileDialog.Filter = "CER Files(*.cer) |*.cer|All files|*.*";
			DialogResult result = this.openFileDialog.ShowDialog();
			switch (result)
			{
				case DialogResult.OK: this.txtCertFilePath.Text = this.openFileDialog.FileName;

					break;
				default: return;
			}
		}
		private void btnBrowseKeyFile_Click(object sender, EventArgs e)
		{
			this.openFileDialog.Filter = "PVT KEY Files(*.pvt) |*.pvt|All files|*.*";
			DialogResult result = this.openFileDialog.ShowDialog();
			switch (result)
			{
				case DialogResult.OK: this.txtCertPrivateKeyFilePath.Text = this.openFileDialog.FileName;

					break;
				default: return;
			}
		}
		#endregion

		#region Private Methods
		private void Setup(CommonForm commonForm)
		{
			try
			{
				//Populate Combo Boxes
				//1. Users
				cmbAuthorization.Items.Clear();
				cmbAuthorization.ImageList = commonForm.ImageList;
				lock (commonForm.Database.Users.SyncRoot)
				{
					foreach (User user in commonForm.Database.Users)
						cmbAuthorization.Items.Add(new ComboBoxWithImagesItem(user.Name,commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.User)));
				}
				
				//2. Certificate Souce Type
				cmbCertSource.DataSource = Enum.GetValues(typeof(CertificateSource));
				cmbCertSource.SelectedIndex = (int)CertificateSource.SelfSigned;

				//3. Display correct Panel
				panelSelfSigned.Enabled = true;
				panelFile.Enabled = false;

				//4. Set Expiry date to one year after current date (Default Behaviour)
				txtEndDate.Value = DateTime.Now.AddYears(1);

				//5. File Dialog filter
				

       			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
				{
					Certificate cert = null;
					lock (commonForm.Database.Certificates.SyncRoot)
					{
						cert = commonForm.Database.Certificates[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
					}
					SetupForEditOperation(cert);
				}
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private void SetupForEditOperation(Certificate cert)
		{
			//Name
			txtName.Text = cert.Name;
			txtName.Enabled = false; //Can't modify the name.

			//Authorization
			cmbAuthorization.Enabled = false; //Can't edit Authorization
			try
			{
				if (cert.Owner.Length != 0)
				{
					cmbAuthorization.SelectedItem = cert.Owner;

				}
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbAuthorization.SelectedIndex = -1;
			}

			//Start - End date
			txtStartDate.Enabled = false;
			txtEndDate.Enabled = false;
			try
			{
				txtStartDate.Value = cert.StartDate;
				txtEndDate.Value = cert.ExpirationDate;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				txtStartDate.Value = DateTime.Now;
				txtEndDate.Value = DateTime.Now.AddYears(1);
			}

			//Subject
			txtCertSubject.Enabled = false;
			try
			{
				txtCertSubject.Text = cert.Subject;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				txtCertSubject.Text = string.Empty;
			}

			//TODO: Certificate Edit operation is not complete.
			txtEncryptPassword.Enabled = false;
			cmbCertSource.Enabled = false;
		}
		private bool ValidateControls()
		{
			bool result = true;

			if (txtName.Text.Length == 0)
			{
				result = false;
				throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "Certificate"));
			}

			if ((CertificateSource)cmbCertSource.SelectedItem == CertificateSource.SelfSigned)
			{
				if (txtCertSubject.Text.Length == 0)
				{
					result = false;
					throw new InvalidArgumentException(Resources.Exceptions.ObjectMaintenance_InvalidCertSubject);
				}
			}
			return result;
		}
		#endregion
	}
}
