//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Collections.Specialized;
using System.Xml;
using System.Windows.Forms;

using Digitaldeposit.Ssbm.ObjectMaintenance.Forms;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.CustomControls;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

namespace Digitaldeposit.Ssbm.ObjectMaintenance.UserControls
{
	/// <summary>
	/// Takes care of all the UI related operation to create a Service Broker Queue object.
	/// </summary>
	public partial class QueueMaintenance : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		public QueueMaintenance(CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			InitializeComponent();
			Setup(commonForm);
		}
		#endregion

		#region Private Methods
		private void Setup(CommonForm commonForm)
		{
			//Populate all the combo boxes
			//1. FileGroups
			cmbFileGroup.Items.Add("[DEFAULT]");
			lock (commonForm.Database.FileGroups.SyncRoot)
			{
				foreach (FileGroup fg in commonForm.Database.FileGroups)
				{
					cmbFileGroup.Items.Add(fg.Name);
				}
			}

			//2. Populate all the databases
			cmbDatabase.Items.Clear();
			cmbDatabase.ImageList = commonForm.ImageList;
			cmbDatabase.Items.Add(new ComboBoxWithImagesItem(string.Empty, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.Database)));
			lock (commonForm.Server.Databases.SyncRoot)
			{
				foreach (Database database in commonForm.Server.Databases)
				{
					ComboBoxWithImagesItem item = new ComboBoxWithImagesItem(database.Name, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.Database));
					item.Tag = database;
					cmbDatabase.Items.Add(item);
				}
			}
			cmbDatabase.SelectedItem = commonForm.Database;

			//3. Populate valid stored procedures from the selected database
			cmbStoredProc.Items.Clear();
			cmbStoredProc.ImageList = commonForm.ImageList;
			cmbStoredProc.Items.Add(new ComboBoxWithImagesItem(string.Empty, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.ActivationSproc)));
			lock (commonForm.Database.StoredProcedures.SyncRoot)
			{
				foreach (StoredProcedure sp in commonForm.Database.StoredProcedures)
				{
					ComboBoxWithImagesItem item = new ComboBoxWithImagesItem(sp.Name, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.ActivationSproc));
					item.Tag = sp;
					cmbStoredProc.Items.Add(item);
				}
			}

			//4. Populate all the users.
			cmbUsers.Items.Clear();
			cmbUsers.ImageList = commonForm.ImageList;
			cmbUsers.Items.Add(new ComboBoxWithImagesItem(string.Empty, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.User)));
			lock (commonForm.Database.Users.SyncRoot)
			{
				foreach (User user in commonForm.Database.Users)
				{
					cmbUsers.Items.Add(new ComboBoxWithImagesItem(user.Name, commonForm.ImageList.Images.IndexOfKey(ProjectImagesConstants.User)));
				}
			}

			//Visibility & Accessebility
			gbActivation.Enabled = false;

			//Load default values if operation is Alter
			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
			{
				SetupForEditOperation(commonForm.Server, commonForm.Database, commonForm.Database.ServiceBroker.Queues[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText]);
			}
		}
		private void SetupForEditOperation(Server server, Database db, ServiceQueue sq)
		{
			//Name
			txtName.Text = sq.Name;
			txtName.Enabled = false; //Can't modify the name.

			//Status
			try
			{
				chkStatus.Checked = sq.IsEnqueueEnabled;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				chkStatus.Checked = false;
			}

			//Retnetion
			try
			{
				chkRetention.Checked = sq.IsRetentionEnabled;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				chkRetention.Checked = false;
			}

			//Activation
			try
			{
				chkActivation.Checked = sq.IsActivationEnabled;

				//Stored Proc Database
				if (sq.ProcedureDatabase.Length != 0)
				{
					db = server.Databases[sq.ProcedureDatabase];
				}
				cmbDatabase.SelectedItem = db;

				//Stored Proc
				if (sq.ProcedureName.Length != 0)
				{
					StoredProcedure sp = db.StoredProcedures[sq.ProcedureName];

					cmbStoredProc.SelectedItem = sp;
				}
			}
			catch (PropertyCannotBeRetrievedException)
			{
				chkActivation.Checked = false;
				cmbDatabase.SelectedIndex = -1;
				cmbStoredProc.SelectedIndex = -1;
			}

			//Maximum Readers
			try
			{
				txtMaxActivation.Text = sq.MaxReaders.ToString();
			}
			catch (PropertyCannotBeRetrievedException)
			{
				txtMaxActivation.Text = string.Empty;
			}

			//Execution Context
			try
			{
				switch (sq.ActivationExecutionContext)
				{
					case ActivationExecutionContext.Self: rbSelf.Checked = true; break;
					case ActivationExecutionContext.Owner: rbOwner.Checked = true; break;
					case ActivationExecutionContext.ExecuteAsUser: rbUser.Checked = true;
						cmbUsers.Text = sq.ExecutionContextPrincipal; break;
				}
			}
			catch (PropertyCannotBeRetrievedException)
			{
				rbSelf.Checked = true;
				cmbUsers.SelectedIndex = -1;
			}

			try
			{
				cmbFileGroup.Text = sq.FileGroup;
				cmbFileGroup.Enabled = false; //Can't modify File Group.
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbFileGroup.Enabled = false;
			}

		}
		private bool ValidateControls()
		{
			bool result = true;

			if (txtName.Text.Length == 0)
			{
				result = false;
				throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "Queue"));
			}

			if (chkActivation.Checked)
			{
				if (cmbDatabase.Text.Length == 0)
				{
					result = false;
					throw new InvalidArgumentException(Resources.Exceptions.ObjectMaintenance_InvalidDatabase);
				}

				if (cmbStoredProc.Text.Length == 0)
				{
					result = false;
					throw new InvalidArgumentException(Resources.Exceptions.ObjectMaintenance_InvalidStoredProcedure);
				}
			}

			return result;
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from form to populate ServiceQueue object with corresponding UI data.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <returns>A populated ServiceQueue object which can then undergo any SQL Operation like Create() or Alter()</returns>
		public ServiceQueue Populate(CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "Queue Maintenance"));

			ServiceQueue sq;

			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
				sq = commonForm.Database.ServiceBroker.Queues[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
			else
			{
				sq = new ServiceQueue();
				sq.Name = txtName.Text;
				sq.Parent = commonForm.Database.ServiceBroker;
			}

			if (ValidateControls())
			{
				//Status
				sq.IsEnqueueEnabled = chkStatus.Checked;
				
				//Retention
				sq.IsRetentionEnabled = chkRetention.Checked;

				//Activation
				if (chkActivation.Checked)
				{
					sq.IsActivationEnabled = chkActivation.Checked;

					//Database
					sq.ProcedureDatabase = cmbDatabase.Text;

					//Stored proc
					ComboBoxWithImagesItem item = cmbStoredProc.SelectedItem as ComboBoxWithImagesItem;
					if (item != null)
					{
						StoredProcedure sp = (StoredProcedure)item.Tag;
						sq.ProcedureName = sp.Name;
						sq.ProcedureSchema = sp.Schema;
					}

					//Max readers
					if (txtMaxActivation.Text.Length != 0)
						sq.MaxReaders = short.Parse(txtMaxActivation.Text);
					else
						sq.MaxReaders = 0;

					//User
					if (rbOwner.Checked)
						sq.ActivationExecutionContext = ActivationExecutionContext.Owner;
					else if (rbUser.Checked)
					{
						if (cmbUsers.SelectedText.Length != 0) //if no user selected, will default to self.
						{
							sq.ActivationExecutionContext = ActivationExecutionContext.ExecuteAsUser;
							sq.ExecutionContextPrincipal = cmbUsers.Text;
						}
					}
				}

				//File group
				if (cmbFileGroup.Text != "[DEFAULT]" && cmbFileGroup.Text.Length != 0)
					sq.FileGroup = cmbFileGroup.Text;

				return sq;
			}

			return null;

		}
		#endregion

		#region Event Handlers
		private void chkActivation_CheckedChanged(object sender, EventArgs e)
		{
			if (chkActivation.Checked)
			{
				gbActivation.Enabled = true;
			}
			else
			{
				gbActivation.Enabled = false;
			}
		}
		private void cmbDatabase_SelectedIndexChanged(object sender, EventArgs e)
		{
			try
			{
				cmbStoredProc.Items.Clear();
				ImageList imageList = ImageListHelper.GetServiceBrokerImages();

				if (cmbDatabase.Text.Length != 0)
				{
					ComboBoxWithImagesItem item = cmbDatabase.SelectedItem as ComboBoxWithImagesItem;
					Database db = (Database)item.Tag;

					//When database is changed, populate corresponding stored procs
					cmbStoredProc.Items.Clear();
					cmbStoredProc.Items.Add(new ComboBoxWithImagesItem(string.Empty, imageList.Images.IndexOfKey(ProjectImagesConstants.ActivationSproc)));
					foreach (StoredProcedure sp in db.StoredProcedures)
					{
						item = new ComboBoxWithImagesItem(sp.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.ActivationSproc));
						item.Tag = sp;
						cmbStoredProc.Items.Add(item);
					}
					
					//When database is changed, populate corresponding users
					cmbUsers.Items.Clear();
					cmbUsers.Items.Add(new ComboBoxWithImagesItem(string.Empty, imageList.Images.IndexOfKey(ProjectImagesConstants.User)));
					foreach (User user in db.Users)
						cmbUsers.Items.Add(new ComboBoxWithImagesItem(user.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.User)));
				}
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		#endregion
	}
}
