//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Collections.Specialized;
using System.Xml;
using System.Windows.Forms;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.ObjectMaintenance.Forms;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.CustomControls;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.Manager;

namespace Digitaldeposit.Ssbm.ObjectMaintenance.UserControls
{
	/// <summary>
	/// Takes care of all the UI related operation to create a Service Broker RemoteServiceBinding object.
	/// </summary>
	public partial class RemoteServiceBidningMaintenance : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <param name="servers">List of servers to populate the treeview, based on the object explorer added servers.</param>
		public RemoteServiceBidningMaintenance(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm, Server[] servers)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			if (servers == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Servers Array", "ContractMaintenance"));

			InitializeComponent();

			Setup(commonForm, servers);
		}
		#endregion
	
		#region Private Methods
		private void Setup(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm, Server[] servers)
		{
			try
			{
				//Assign imagelist to required objects.
				ImageList imageList = commonForm.ImageList;
				tvRemoteServices.ImageList = imageList;
				cmbAuthorization.ImageList = imageList;
				cmbCertificateUser.ImageList = imageList;

				//Populate Combo Boxes
				//1. Users and Roles
				cmbAuthorization.Items.Clear();
				cmbAuthorization.Items.Add(new ComboBoxWithImagesItem(string.Empty, imageList.Images.IndexOfKey(ProjectImagesConstants.User)));
				lock (commonForm.Database.Users.SyncRoot)
				{
					foreach (User u in commonForm.Database.Users)
					{
						cmbAuthorization.Items.Add(new ComboBoxWithImagesItem(u.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.User)));
						cmbCertificateUser.Items.Add(new ComboBoxWithImagesItem(u.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.User)));
					}
				}
				lock (commonForm.Database.Roles.SyncRoot)
				{
					foreach (DatabaseRole r in commonForm.Database.Roles)
						cmbAuthorization.Items.Add(new ComboBoxWithImagesItem(r.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.UserGroup)));
				}

				//If alter operation
				if (commonForm.Operation == SsbMaintenanceOperation.Alter)
				{
					RemoteServiceBinding rsb = commonForm.Database.ServiceBroker.RemoteServiceBindings[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
					SetupForEditOperation(rsb);
				}

				//Start Asynchronous process of getting Remote Services as XML.
				tvRemoteServices.Nodes.Add(TreeViewHelper.DummyTreeNode(TreeViewHelper.POPULATING_CONTENT));
				bgw_PopulateRemoteServices.RunWorkerAsync(servers);
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private bool ValidateControls()
		{
			bool result = true;

			if (txtName.Text.Length == 0)
			{
				result = false;
				throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "Remote Service Binding"));
			}

			if (lblSelectedRemoteService.Text .Length == 0)
			{
				result = false;
				throw new InvalidArgumentException (Resources.Exceptions.ObjectMaintenance_InvalidService); 
			}

			if (cmbCertificateUser.Text.Length == 0)
			{
				result = false;
				throw new InvalidArgumentException(Resources.Exceptions.ObjectMaintenance_InvalidServiceUser);
			}

			return result;
		}
		private void SetupForEditOperation(RemoteServiceBinding rsb)
		{
			//Name
			txtName.Text = rsb.Name;
			txtName.Enabled = false; //Can't modify the name.

			//Authorization
			cmbAuthorization.Enabled = false; //Can't modify authorization.
			try
			{
				cmbAuthorization.Text = rsb.Owner;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbAuthorization.SelectedIndex = -1;
			}

			//Remote Service
			try
			{
				lblSelectedRemoteService.Text = rsb.RemoteService;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				lblSelectedRemoteService.Text = string.Empty;
			}

			//Anonymous Access
			try
			{
				chkAnonymous.Checked = rsb.IsAnonymous;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				chkAnonymous.Checked = false;
			}

			try
			{
				cmbCertificateUser.Text = rsb.CertificateUser;
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbCertificateUser.SelectedIndex = -1;
			}

			tvRemoteServices.Enabled = false;
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from form to populate RemoteServiceBinding object with corresponding UI data.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <returns>A populated RemoteServiceBinding object which can then undergo any SQL Operation like Create() or Alter()</returns>
		public RemoteServiceBinding Populate(Digitaldeposit.Ssbm.ObjectMaintenance.Forms.CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "Remote Service Binding Maintenance"));

			RemoteServiceBinding rsb;

			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
				rsb = commonForm.Database.ServiceBroker.RemoteServiceBindings[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
			else
			{
				rsb = new RemoteServiceBinding();
				rsb.Name = txtName.Text;
				rsb.Parent = commonForm.Database.ServiceBroker;

				if (cmbAuthorization.Text.Length != 0)
					rsb.Owner = cmbAuthorization.Text;

				rsb.RemoteService = lblSelectedRemoteService.Text;
			}

			rsb.CertificateUser = cmbCertificateUser.Text;

			rsb.IsAnonymous = chkAnonymous.Checked;

			if (ValidateControls())
			{
				return rsb;
			}

			return null;
		}
		#endregion

		#region Event Handlers
		private void tvRemoteServices_AfterSelect(object sender, TreeViewEventArgs e)
		{
			TreeNode selectedNode = tvRemoteServices.SelectedNode;

			//Make sure user can select the text only at Service Node level

			if (selectedNode != null)
			{
				if (selectedNode.Tag != null && (selectedNode.Tag.GetType() == typeof(XmlElement) || selectedNode.Tag.GetType() == typeof(XmlNode)))
				{
					XmlNode xn = (XmlNode)selectedNode.Tag;
					if (xn.Name == SsbmObjectType.Service.ToString())
					{
						lblSelectedRemoteService.Text = selectedNode.Text;
					}
				}
			}
		}
		private void bgw_PopulateRemoteServices_DoWork(object sender, DoWorkEventArgs e)
		{
			// This method will run on a thread other than the UI thread.
			// Be sure not to manipulate any Windows Forms controls created
			// on the UI thread from this method.

			if (bgw_PopulateRemoteServices.CancellationPending)
				e.Cancel = true;
			else
			{
				ServiceBrokerManager sbm = new ServiceBrokerManager();
				e.Result = sbm.ConstructRemoteServicesXml((Server[])e.Argument);
			}
		}
		private void bgw_PopulateRemoteServices_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			if (!e.Cancelled)
			{
				if (e.Error == null) //This is not working properly, as documented in msdn.
				{
					object result = e.Result;

					Exception returnException = result as Exception; //so our own logic to handle exception.

					if (returnException == null)
					{
						try
						{
							string ServerInfoXml = (string)e.Result;

							//Load the Object Explorer treeview
							XmlDocument xmlDocument = new XmlDocument();
							xmlDocument.LoadXml(ServerInfoXml);

							tvRemoteServices.BeginUpdate();	// Suppress repainting the TreeView until all the objects have been created.
							TreeNode tNode;
							XmlNode xNode;

							//Find the correct status node set by the Async Start process
							TreeNode[] tnc = tvRemoteServices.Nodes.Find(TreeViewHelper.DUMMYNODE, true);
							tvRemoteServices.Nodes[tnc[0].Index].Remove();
							tvRemoteServices.Nodes.Add(new TreeNode(xmlDocument.DocumentElement.Name));
							tNode = tvRemoteServices.TopNode;
							xNode = xmlDocument.SelectSingleNode(xmlDocument.DocumentElement.Name);

							TreeViewHelper tvh = new TreeViewHelper();
							tvh.AddNode(ref xNode, ref tNode, false, 0, null);

							tvRemoteServices.Nodes[0].Expand();
							tvRemoteServices.EndUpdate();
						}
						catch (Exception ex)
						{
							ExceptionMessageBox emb = new ExceptionMessageBox(ex);
							emb.Show(this);
						}
					}
					else
					{
						ExceptionMessageBox emb = new ExceptionMessageBox(returnException);
						emb.Show(this);
					}
				}
			}
		}
		#endregion
	}
}
