//######################################################
//Author:	Saravana Kumar
//Company:	Digital Deposit Ltd
//Purpose:	Connected System Developer Comp. 2005
//Email:	saravana.kumar@digitaldeposit.co.uk
//######################################################
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Collections.Specialized;
using System.Xml;
using System.Windows.Forms;

using DevComponents.DotNetBar;

using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Smo.Broker;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.MessageBox;

using Digitaldeposit.Ssbm.ObjectMaintenance.Forms;
using Digitaldeposit.Ssbm.Constants;
using Digitaldeposit.Ssbm.Common.Enums;
using Digitaldeposit.Ssbm.Common;
using Digitaldeposit.Ssbm.CustomControls;

namespace Digitaldeposit.Ssbm.ObjectMaintenance.UserControls
{
	/// <summary>
	/// Takes care of all the UI related operation to create a Database User object.
	/// </summary>
	public partial class UserMaintenance : UserControl
	{
		#region Constructor
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		public UserMaintenance(CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			InitializeComponent();
			
			//Prepare the form.
			Setup(commonForm);
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Called from form to populate User object with corresponding UI data.
		/// </summary>
		/// <param name="commonForm">Instance of CommonForm object, which contains all the relevant information like Server, Database Imagelist etc.</param>
		/// <returns>A populated User object which can then undergo any SQL Operation like Create() or Alter()</returns>
		public User Populate(CommonForm commonForm)
		{
			if (commonForm == null)
				throw new ArgumentException(String.Format(Digitaldeposit.Ssbm.Resources.Exceptions.ObjectMaintenance_InvalidArgument, "Common Form", "ContractMaintenance"));

			User user;

			if (commonForm.Operation == SsbMaintenanceOperation.Alter)
			{
				lock (commonForm.Database.Users.SyncRoot)
				{
					user = commonForm.Database.Users[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
				}
			}
			else
			{
				user = new User();
				user.Parent = commonForm.Database;
				user.Name = txtName.Text;

				if (cmbLogin.Text.Length != 0)
					user.Login = cmbLogin.Text;

				if (cmbCertificate.Text.Length != 0)
					user.Certificate = cmbCertificate.Text;

			}
		
			if (cmbSchemaCollection.Text.Length != 0)
				user.DefaultSchema = cmbSchemaCollection.Text;


			if (ValidateControls())
			{
				return user;
			}
			return null;
		}
		#endregion

		#region Private Methods
		private void Setup(CommonForm commonForm)
		{
			try
			{
				ImageList imageList = commonForm.ImageList;

				//Populate Combo Boxes
				// 1. Logins
				cmbLogin.Items.Clear();
				cmbLogin.ImageList = commonForm.ImageList;
				lock (commonForm.Server.Logins.SyncRoot )
				{
					foreach (Login login in commonForm.Server.Logins)
						cmbLogin.Items.Add(new ComboBoxWithImagesItem(login.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.Login)));
				}
				if (cmbLogin.Items.Count > 0)
					cmbLogin.SelectedIndex = 0;

				// 2. Certificates
				cmbCertificate.Items.Clear();
				cmbCertificate.ImageList = imageList;
				lock (commonForm.Database.Certificates.SyncRoot)
				{
					foreach (Certificate cert in commonForm.Database.Certificates)
						cmbCertificate.Items.Add(new ComboBoxWithImagesItem(cert.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.Certificates)));
				}

				// 3. Schemas
				cmbSchemaCollection.Items.Clear();
				cmbSchemaCollection.ImageList = imageList;
				cmbSchemaCollection.Items.Add(string.Empty); //Schema can be left to default.
				lock(commonForm.Database.Schemas.SyncRoot)
				{
					foreach (Schema sch in commonForm.Database.Schemas)
						cmbSchemaCollection.Items.Add(new ComboBoxWithImagesItem(sch.Name, imageList.Images.IndexOfKey(ProjectImagesConstants.SchemaCollection)));
				}

				if (this.cmbSchemaCollection.Items.Contains("dbo"))
					this.cmbSchemaCollection.SelectedItem = "dbo";
			
				if (commonForm.Operation == SsbMaintenanceOperation.Alter)
				{
					User user = null;

					lock (commonForm.Database.Users.SyncRoot )
					{
						user = commonForm.Database.Users[commonForm.InfoNode.Attributes.GetNamedItem(XmlNodeAttributeConstants.DISPLAY_NAME).InnerText];
					}
					SetupForEditOperation(user);
				}
			}
			catch (Exception ex)
			{
				ExceptionMessageBox emb = new ExceptionMessageBox(ex);
				emb.Show(this);
			}
		}
		private void SetupForEditOperation(User user)
		{
			//For user only Name and Schema can be altered
			txtName.Text = user.Name;
			//txtName.Enabled = false;

			//Login
			cmbLogin.Enabled = false;
			try
			{
				if (user.Login.Length != 0)
				{
					cmbLogin.SelectedIndex = cmbLogin.FindString(user.Login);
				}
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbLogin.SelectedIndex = -1;
			}

			//Certificate
			cmbCertificate.Enabled = false;
			try
			{
				if (user.Certificate.Length != 0)
				{
					cmbCertificate.SelectedIndex = cmbCertificate.FindString(user.Certificate);
				}
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbCertificate.SelectedIndex = -1;
			}

			try
			{
				if (user.DefaultSchema.Length != 0)
					cmbSchemaCollection.SelectedIndex = cmbSchemaCollection.FindString(user.DefaultSchema);
			}
			catch (PropertyCannotBeRetrievedException)
			{
				cmbSchemaCollection.SelectedIndex = -1;
			}
		}
		private bool ValidateControls()
		{
			bool result = true;

			if (txtName.Text.Length == 0)
			{
				result = false;
				throw new InvalidArgumentException(String.Format(Resources.Exceptions.ObjectMaintenance_InvalidName, "User"));
			}

			return result;
		}
		#endregion
	}
}
